/*
 *  Extensions to C++ Standard Template Library
 *
 *  Macros for commonly used code
 */

#ifndef STL_EXT_H
#define STL_EXT_H

/*! Macro for visiting each element of a STL container, in a for loop.
 *  Example of usage:
 *    foreach (listeners, i)
 *      (*i)->notify();
 */
#define foreach( c, i )\
  typedef __typeof__( c ) c##_CONTAINERTYPE;\
  for( c##_CONTAINERTYPE::iterator i = c.begin(); i != c.end(); ++i )

#define foreach_const( c, i )\
  typedef __typeof__( c ) c##_CONTAINERTYPE;\
  for( c##_CONTAINERTYPE::const_iterator i = c.begin(); i != c.end(); ++i )

/*! Macro for calling one method of all elements of a STL container, in a for loop.
 *  Example of usage:
 *    forall (listeners, notify());
 */
#define forall( c, m )\
  typedef __typeof__( c ) c##_CONTAINERTYPE;\
  for( c##_CONTAINERTYPE::iterator i = c.begin(); i != c.end(); ++i )\
    (*i).m;

#endif /* STL_EXT_H */
